//+=============================================================================
//
// file :         Keithley2701.cpp
//
// description :  C++ source for the Keithley2701 and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                Keithley2701 are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: claudio $
//
// $Revision: 1.12 $
//
// $Log: Keithley2701.cpp,v $
// Revision 1.12  2017-03-29 07:25:16  claudio
// ticket 1934: tango-9
//
// Revision 1.11  2013-08-23 10:44:58  graziano
// Device_4Impl
//
// Revision 1.10  2013-08-09 12:18:46  graziano
// fixed
//
// Revision 1.9  2010/12/15 08:57:48  graziano
// fixed socket timeout
//
// Revision 1.8  2009/11/19 07:13:56  graziano
// replaced ATTR_WARNING with ATTR_INVALID
//
// Revision 1.7  2009/11/13 07:56:42  graziano
// bug fixing
//
// Revision 1.6  2009/10/21 10:16:42  graziano
// fixed loss of configuration
//
// Revision 1.5  2009/09/25 12:51:41  graziano
// changed timeout
//
// Revision 1.4  2009/09/24 07:43:25  graziano
// bug fixing
//
// Revision 1.3  2009/09/23 07:23:35  graziano
// bug fixing
//
// Revision 1.2  2009/09/22 12:46:34  graziano
// bug fixing
//
// Revision 1.1  2009/09/22 12:10:18  graziano
// first commit
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name|  Method name
//	----------------------------------------
//  State   |  dev_state()
//  Status  |  dev_status()
//  Start   |  start()
//  Stop    |  stop()
//
//===================================================================


#include <tango.h>
#include <Keithley2701.h>
#include <Keithley2701Class.h>
#include <math.h>	//for modf
#include <algorithm>	//for find
#include "readthread.h"

namespace Keithley2701_ns
{

//+----------------------------------------------------------------------------
//
// method : 		Keithley2701::Keithley2701(string &s)
// 
// description : 	constructor for simulated Keithley2701
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
Keithley2701::Keithley2701(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

Keithley2701::Keithley2701(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

Keithley2701::Keithley2701(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		Keithley2701::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void Keithley2701::delete_device()
{
	//	Delete device's allocated object
	DEBUG_STREAM << "Keithley2701::delete_device: entering..." << endl;
	abortflag = true;
	do
	{
		usleep(100000);
	}while(!terminated);
	usleep(100000);
	delete mutex;
	delete devsocket;
	INFO_STREAM << "Keithley2701::delete_device: after delete devsocket" << endl;
#ifdef _OLD_ATTR
	for(vector<string>::iterator it = added_attr.begin(); it != added_attr.end(); it++)
	{
		try 
		{
			remove_attribute(*it, /*delete object*//*true*/false);
			INFO_STREAM << "Keithley2701::delete_device: Removed attribute " << *it << endl;
		}
		catch(...)
		{
			WARN_STREAM << "Keithley2701::delete_device: Could not remove attribute " << *it << " !" << endl;
		}		
	}
	/*try 
	{
		added_attr.clear();
		DEBUG_STREAM << "Keithley2701::delete_device: added_attr cleared !" << endl;
	}
	catch(...)
	{
		WARN_STREAM << "Keithley2701::delete_device: Could not clear added_attr !" << endl;
	}*/
	DEBUG_STREAM << "Keithley2701::delete_device: added_attr.size=" << added_attr.size() << endl;
	try 
	{
		chan_map.clear();
		scanString.clear();
		initString.clear();
		channels.clear();
		DEBUG_STREAM << "Keithley2701::delete_device: chan_map cleared !" << endl;
	}
	catch(...)
	{
		WARN_STREAM << "Keithley2701::delete_device: Could not clear chan_map !" << endl;
	}	
	for(int i=0; i< num_channels; i++)
	{
		//delete attr[i];	//deleted in remove_attribute
	}
#endif
	delete [] attr_val_read;
	DEBUG_STREAM << "Keithley2701::delete_device: after delete [] attr_val_read;" << endl;
	try 
	{
		chan_map.clear();
		DEBUG_STREAM << "Keithley2701::delete_device: chan_map cleared !" << endl;
		scanString.clear();
		DEBUG_STREAM << "Keithley2701::delete_device: scanString cleared !" << endl;
		initString.clear();
		DEBUG_STREAM << "Keithley2701::delete_device: initString cleared !" << endl;
		channels.clear();
		DEBUG_STREAM << "Keithley2701::delete_device: chan_map cleared !" << endl;
	}
	catch(...)
	{
		WARN_STREAM << "Keithley2701::delete_device: Could not clear chan_map !" << endl;
	}
	//sleep(1);
	//Tango::leavefunc();
	DEBUG_STREAM << "Keithley2701::delete_device: exiting..." << endl;
}

//+----------------------------------------------------------------------------
//
// method : 		Keithley2701::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void Keithley2701::init_device()
{
	INFO_STREAM << "Keithley2701::Keithley2701() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();

	set_change_event("state", true, false);		//attribute name, implemented, detect, if not setting detect=false have to set abs_change or rel_change for each attribute

	Tango::Database *db = new Tango::Database();
	Tango::DbData db_data;	
	try {
		db_data.push_back(Tango::DbDatum("Readtimeout"));
		string in_prop("timeout");
//		devserial->get_property(in_prop, db_data);
		db->get_device_property(conndevice,db_data);
		db_data[0] >> socket_timeout;
		delete db;
	}
	catch(Tango::DevFailed e) {
		string temp;
		temp = e.errors[0].desc;
		WARN_STREAM << "MenloSYNCRO::MenloSYNCRO() error getting timeout property from socket device: " << temp << endl;
		delete db;		
	}
	DEBUG_STREAM << "Read socket timeout="<<socket_timeout<< endl;
	if(socket_timeout < 0 || socket_timeout > 30000)
		socket_timeout = 3000;
	/* connect to socket device */
	devsocket = new Tango::DeviceProxy(conndevice);	
	scanning = true;
	connection_status = 0;
	abortflag = false;
	terminated = false;
	timeout = 0;
	stringstream tmp_name;
	num_channels = 0;
	for(vector<string>::iterator it = channels.begin(); it != channels.end(); it++)
	{
		//DEBUG_STREAM << "Keithley2701::Keithley2701(): added_attr.size=" << added_attr.size() << endl;
		size_t pos = it->find(",");
		if(pos != string::npos)
		{
			string num = it->substr(0, pos);
			string name = it->substr(pos+1, it->length()-1 );
			string unit = "";
			pos = name.find(",");
			if(pos != string::npos)
			{
				unit = name.substr(pos+1, name.length()-1 );
				name = name.substr(0, pos);
			}
			stringstream tmp;
			int num_ch;
			tmp << num;
			tmp >> num_ch; 
			DEBUG_STREAM << "Keithley2701::Keithley2701() found channel config string= " << num_ch << " - " <<  name  << " ; " << unit << endl;
			struct timeval curr_time;
			//gettimeofday(&curr_time,NULL);
			curr_time.tv_sec = 0;
			data_t c_t = {0.0,name,curr_time};
			chan_map.insert(make_pair(num_ch,c_t));

#ifdef _OLD_ATTR
			try
			{
				if(find(added_attr.begin(),added_attr.end(),name) == added_attr.end())
				{
					attr[num_channels] = new ScalarAttributeDouble(name,Tango::READ,this, num_ch);
					add_attribute(attr[num_channels]);
					added_attr.push_back(name);
					INFO_STREAM << "Keithley2701::init_device: Added attribute " << name << endl;
				}
				else
				{
					attr[num_channels]->init(name,Tango::READ,this, num_ch);
					INFO_STREAM << "Keithley2701::init_device: before add_attribute " << name << endl;	
					add_attribute(attr[num_channels]);
					INFO_STREAM << "Keithley2701::init_device: Init attribute " << name << endl;				
				}
			}
			catch(...)
			{
				WARN_STREAM << "Keithley2701::init_device: Could not add attribute " << name  << " !" << endl;
			}
#endif
			num_channels++;
		}
			
	}
	timeout = int(2*(2000 + th_sleep + num_channels*900));
	DEBUG_STREAM << "Keithley2701::init_device: allocating double * " << num_channels  << " !" << endl;
	attr_val_read = new double[num_channels];
	data_inited = false;
	//read_time = 0;
	mutex = new omni_mutex();
	Configure();
	readloop = new readthread(this);
	readloop->start();
}


//+----------------------------------------------------------------------------
//
// method : 		Keithley2701::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void Keithley2701::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Conndevice"));
	dev_prop.push_back(Tango::DbDatum("Th_sleep"));
	dev_prop.push_back(Tango::DbDatum("Wr_sleep"));
	dev_prop.push_back(Tango::DbDatum("ScanString"));
	dev_prop.push_back(Tango::DbDatum("Channels"));
	dev_prop.push_back(Tango::DbDatum("AttrName"));
	dev_prop.push_back(Tango::DbDatum("InitString"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	Keithley2701Class	*ds_class =
		(static_cast<Keithley2701Class *>(get_device_class()));
	int	i = -1;

	//	Try to initialize Conndevice from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  conndevice;
	else {
		//	Try to initialize Conndevice from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  conndevice;
	}
	//	And try to extract Conndevice value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  conndevice;

	//	Try to initialize Th_sleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  th_sleep;
	else {
		//	Try to initialize Th_sleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  th_sleep;
	}
	//	And try to extract Th_sleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  th_sleep;

	//	Try to initialize Wr_sleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  wr_sleep;
	else {
		//	Try to initialize Wr_sleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  wr_sleep;
	}
	//	And try to extract Wr_sleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  wr_sleep;

	//	Try to initialize ScanString from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  scanString;
	else {
		//	Try to initialize ScanString from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  scanString;
	}
	//	And try to extract ScanString value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  scanString;

	//	Try to initialize Channels from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  channels;
	else {
		//	Try to initialize Channels from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  channels;
	}
	//	And try to extract Channels value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  channels;
	
	//get_db_device()->put_property(dev_prop);	//TODO: needed? (from howto)

	//	Try to initialize AttrName from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  attrName;
	else {
		//	Try to initialize AttrName from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  attrName;
	}
	//	And try to extract AttrName value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  attrName;

	//	Try to initialize InitString from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  initString;
	else {
		//	Try to initialize InitString from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  initString;
	}
	//	And try to extract InitString value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  initString;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		Keithley2701::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void Keithley2701::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		Keithley2701::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void Keithley2701::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "Keithley2701::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		Keithley2701::read_ScalarAttribute
// 
// description : 	Extract real attribute values for ScalarAttribute acquisition result.
//
//-----------------------------------------------------------------------------
void Keithley2701::read_ScalarAttribute(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "Keithley2701::read_ScalarAttribute(Tango::Attribute &attr) entering... "<< endl;
	int num_ch=-1;
	int index = 0;
	char tmp[10];
	bool b_timeout = false;
	struct timeval read_time;
	for(map<int,data_t>::iterator it=chan_map.begin(); it != chan_map.end(); it++)
	{
		if (!strcmp(attr.get_name().c_str(),it->second.name.c_str())) 
		{
			num_ch=it->first;
			attr_val_read[index] = it->second.val;
			read_time = it->second.read_time;
			break;
		}
		index++;
	}
	//DEBUG_STREAM << "Data Returned defined by attr_ScalarAttribute_read["<< num_ch<<"]: " << (*(attr_val_read+index))<< endl;
	//attr.set_value(attr_val_read+index);
	
	if (check_timeout(read_time, timeout) < 0) 
	{        
		DEBUG_STREAM << "Keithley2701::read: "<< attr.get_name() << " ch=" << num_ch << ": data is not refreshed" << endl;
		b_timeout = true;					
	}
	if(get_state() == Tango::FAULT || !data_inited)
	{
		attr.set_quality(Tango::ATTR_INVALID);
	}
	if(!b_timeout)
		attr.set_value_date_quality(&attr_val_read[index],read_time,Tango::ATTR_VALID, 1, 0, false);
	else
		//attr.set_value_date_quality(&attr_val_read[index],read_time,Tango::ATTR_WARNING, 1, 0, false);
		attr.set_quality(Tango::ATTR_INVALID);
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * 				SCAN EXAMPLE
		*RST
		TRAC:CLE
		TRAC:CLE:AUTO ON
		FORM:ELEM READ,TST,CHAN
		CONF:TEMP (@101:108)
		TEMP:TRAN FRTD,(@101:108)
		TEMP:FRTD:TYPE PT100,(@101:108)
		CONF:DCV (@109:110)
		CALC:FORM MXB,(@110)
		CALC:KMATH:MMF 0.001,(@110)
		CALC:KMATH:MBF 0.2,(@110)
		CALC:STAT ON,(@110)
		INIT:CONT OFF
		TRIG:SOUR IMM
		TRIG:COUN 1
		SAMP:COUN 10
		ROUT:SCAN (@101:110)
		ROUT:SCAN:LSEL INT
		READ?
 * 
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
 
//+------------------------------------------------------------------
/**
 *	method:	Keithley2701::start
 *
 *	description:	method to execute "Start"
 *	Start temperature acquisition
 *
 *
 */
//+------------------------------------------------------------------
void Keithley2701::start()
{
	DEBUG_STREAM << "Keithley2701::start(): entering... !" << endl;

	//	Add your own code to control device here
	scanning = true;
	if(get_state() != Tango::RUNNING && get_state() != Tango::FAULT)
	{
		Tango::DevState st = Tango::RUNNING;
		set_state(st);
		push_change_event("state", &st);
	}
	set_status("Acquisition Running");

}

//+------------------------------------------------------------------
/**
 *	method:	Keithley2701::stop
 *
 *	description:	method to execute "Stop"
 *	Sop temperature acquisition
 *
 *
 */
//+------------------------------------------------------------------
void Keithley2701::stop()
{
	DEBUG_STREAM << "Keithley2701::stop(): entering... !" << endl;

	//	Add your own code to control device here
	scanning = false;

	if(get_state() != Tango::ON && get_state() != Tango::FAULT)
	{
		Tango::DevState st = Tango::ON;
		set_state(st);
		push_change_event("state", &st);
	}
	set_status("Acquisition Not Running");
}


int Keithley2701::WriteRead(string cmd, string &response)
{
	//DEBUG_STREAM << "Keithley2701::WriteRead(): entering... " << endl;
	mutex->lock();
	char outbuf[256],
	inbuf[256];
	stringstream temp;
	//string cmd;
	bool ok = false;
	//int nretries = 0;

	//DEBUG_STREAM << "Keithley2701::WriteRead() before Verifying connection status...." << endl;
	if(!connection_status)
	{
		DEBUG_STREAM << "Keithley2701::WriteRead() Verifying connection status...." << endl;
		Tango::DeviceAttribute attr_state;
		Tango::DevState stat;
		try {
			attr_state = devsocket->read_attribute("State");
			attr_state >> stat;
		}
		catch(Tango::DevFailed e) {
			connection_status = 0;
			if(get_state() != Tango::FAULT)
			{
				Tango::DevState st = Tango::FAULT;
				set_state(st);
				push_change_event("state", &st);
			}
			set_status("Failed reading State from socketdevice " + conndevice);
			INFO_STREAM << "Failed reading State from socketdevice " + conndevice << endl;
			mutex->unlock();
			return -1;	
		}
		if(stat == Tango::ON)
		{
			DEBUG_STREAM << "Keithley2701::WriteRead() ...connection status OK!" << endl;
			connection_status = 1;
			if(scanning)
			{
				if(get_state() != Tango::RUNNING)
				{
					Tango::DevState st = Tango::RUNNING;
					set_state(st);
					push_change_event("state", &st);
				}
				set_status("Acquisition Running");
			}
			else
			{
				if(get_state() != Tango::ON)
				{
					Tango::DevState st = Tango::ON;
					set_state(st);
					push_change_event("state", &st);
				}
				set_status("Acquisition Not Running");		
			}
			devsocket->set_timeout_millis((num_channels*900) > socket_timeout ? (num_channels*900) : socket_timeout);  //TODO: verify if enough
		}			
		else
		{
			try {
				DEBUG_STREAM << "Keithley2701::WriteRead(): ...connection status not OK, reconnecting... " << endl;		
				devsocket->command_inout("Reconnect");
				if(scanning)
				{
					if(get_state() != Tango::RUNNING)
					{
						Tango::DevState st = Tango::RUNNING;
						set_state(st);
						push_change_event("state", &st);
					}
					set_status("Acquisition Running");
				}
				else
				{
					if(get_state() != Tango::ON)
					{
						Tango::DevState st = Tango::ON;
						set_state(st);
						push_change_event("state", &st);
					}
					set_status("Acquisition Not Running");		
				}
			}
			catch(Tango::DevFailed e) {	
				if(get_state() != Tango::FAULT)
				{
					Tango::DevState st = Tango::FAULT;
					set_state(st);
					push_change_event("state", &st);
				}
				TangoSys_MemStream out_stream;
				out_stream << "Error reconnecting socket , err = " << e.errors[0].desc << ends;
				string temp_err(e.errors[0].desc);
				/*if(temp_err.find("Reconnect not allowed") != string::npos)
				{
					connection_status=1;
					if(scanning)
					{
						if(get_state() != Tango::RUNNING)
						{
							Tango::DevState st = Tango::RUNNING;
							set_state(st);
							push_change_event("state", &st);
						}
						set_status("Acquisition Running");
					}
					else
					{
						if(get_state() != Tango::ON)
						{
							Tango::DevState st = Tango::ON;
							set_state(st);
							push_change_event("state", &st);
						}
						set_status("Acquisition Not Running");		
					}
					devsocket->set_timeout_millis(num_channels*900);
				}
				else*/
				{
					set_status(out_stream.str());			
					WARN_STREAM  << "Keithley2701::WriteRead(): " << out_stream.str() << endl;
					connection_status = 0;
					mutex->unlock();
					return -1;
					/*Tango::Except::re_throw_exception(e,
						(const char *) "Error reconnecting socket",
						out_stream.str(),
						(const char *) "Keithley2701::WriteRead()", Tango::ERR);*/	
				}
			}
		}
		if(connection_status)	//if just connected
		{
			Tango::DeviceData Dout_tmp;
			string s_tmp;
			try {
				Dout_tmp = devsocket->command_inout("Read");
				Dout_tmp >> s_tmp;
			}
			catch(Tango::DevFailed e) {	
				TangoSys_MemStream out_stream;
				out_stream << "Error emptying socket = " << e.errors[0].desc << ends;			
				INFO_STREAM  << "Keithley2701::WriteRead(): " << out_stream.str() << endl;
				abort_sleep((double)wr_sleep/1000);  //abort_sleep(sec)		
			}
			if(s_tmp != "")
			{
				INFO_STREAM  << "Keithley2701::WriteRead(): read " << s_tmp << " emptying socket" << endl;
			}
		}	
	}


	//sprintf(outbuf,"%s%c",cmd.c_str(),END_CHAR);	
	
		DEBUG_STREAM << "Keithley2701::WriteRead() writing command: " << cmd << endl;
		string s("");//contains the answer to the cmd if any
		Tango::DeviceData Din, Dout;
		Din << cmd;
		if(cmd.find("?") == string::npos)
		{	
			try {
				devsocket->command_inout("Write", Din);
				DEBUG_STREAM  << "Keithley2701::WriteRead(): write! going to sleep" << endl;
				//abort_sleep((double)wr_sleep/1000);  //abort_sleep(sec)			
				//abort_sleep(0.800);  //abort_sleep(sec)
			}
			catch(Tango::DevFailed e) {	
				if(get_state() != Tango::FAULT)
				{
					Tango::DevState st = Tango::FAULT;
					set_state(st);
					push_change_event("state", &st);
				}
				TangoSys_MemStream out_stream;
				out_stream << "Error writing cmd = " << cmd << ", err = " << e.errors[0].desc << ends;
				set_status(out_stream.str());			
				WARN_STREAM  << "Keithley2701::WriteRead(): " << out_stream.str() << endl;
				connection_status = 0;
				mutex->unlock();
				return -1;
			}
		} else
		//if(cmd.find("?") != string::npos)
		{
			DEBUG_STREAM  << "Keithley2701::WriteRead(): going to read response" << endl;

				try {
					Dout = devsocket->command_inout("WriteAndRead",Din);
					gettimeofday(&(global_read_time),NULL);
					Dout >> s;
				}
				catch(Tango::DevFailed e) {
					gettimeofday(&(global_read_time),NULL);
					TangoSys_MemStream out_stream;
					out_stream << "Error reading after write/read cmd = " << cmd << ", err = " << e.errors[0].desc << ends;		
					WARN_STREAM  << "Keithley2701::WriteRead(): " << out_stream.str() << endl;
					abort_sleep(((double)(num_channels*300.0))/1000);  //abort_sleep(sec)		
				}
				if(s == "")
				{
					DEBUG_STREAM  << "Keithley2701::WriteRead(): response empty, retrying" << endl;
					//abort_sleep((double)(num_channels*300.0)/1000);  //abort_sleep(sec)
					try {
						Dout = devsocket->command_inout("Read");
						Dout >> s;
					}
					catch(Tango::DevFailed e) {	
						if(get_state() != Tango::FAULT)
						{
							Tango::DevState st = Tango::FAULT;
							set_state(st);
							push_change_event("state", &st);
						}
						TangoSys_MemStream out_stream;
						out_stream << "Error reading II after read cmd = " << cmd << ", err = " << e.errors[0].desc << ends;
						set_status(out_stream.str());			
						WARN_STREAM  << "Keithley2701::WriteRead(): " << out_stream.str() << endl;
						connection_status = 0;
					}
				}
		}
		
	//DEBUG_STREAM  << "Keithley2701::WriteRead(): read response=" << s << endl;
	response = s;
	mutex->unlock();
	return 0;
				
}

int Keithley2701::Scan()
{
	//DEBUG_STREAM << "Keithley2701::Scan(): entering... " << endl;
	vector<string>::iterator it;
	string resp,cmd;
	for(it=scanString.begin(); it!=scanString.end(); it++)
	{
		cmd = *it;
		WriteRead(cmd,resp);
		DEBUG_STREAM << "Keithley2701::Scan() writing command: " << cmd << " read response=" << resp << endl;
		abort_sleep((double)2.0*wr_sleep/1000);  //abort_sleep(sec)	
	//}
	if(resp.find("RDNG") != string::npos)
	{
		WARN_STREAM << "Keithley2701::Scan() Lost configuration! read " << resp << endl;
		return -1;
	}
	size_t pos_ini=0, pos_fin;
	string val;
	int i=0;
	do
	{
		bool found_val = false;
		bool found_time = false;
		bool found_ch = false;
		double tmp_val;
		double tmp_time;
		double tmp_time_diff;
		int num_ch;
		pos_fin = resp.find(",");
		if(pos_fin != string::npos)
		{
			val = resp.substr(pos_ini, pos_fin-pos_ini);
			resp = resp.substr(pos_fin+1, resp.length()-pos_fin-1 );
			stringstream tmp;
			tmp << val;
			tmp >> tmp_val; 
			DEBUG_STREAM << "Keithley2701::Scan() found val string= " << val << " val number=" <<  tmp_val << endl;//"; now resp=" << resp << endl;
			found_val = true;
		}
		/*else if(resp.length() > 0)
		{
			stringstream tmp;
			tmp << resp;
			tmp >> tmp_val; 
			DEBUG_STREAM << "Keithley2701::Scan() found last val, string= " << resp << " val number=" <<  tmp_val << endl;
			//found_val = true;
		}*/
		
		pos_fin = resp.find(",");
		if(pos_fin != string::npos)
		{
			val = resp.substr(pos_ini, pos_fin-pos_ini);
			resp = resp.substr(pos_fin+1, resp.length()-pos_fin-1 );
			stringstream tmp;
			tmp << val;
			tmp >> tmp_time_diff; 
			DEBUG_STREAM << "Keithley2701::Scan() found time string= " << val << " time number=" <<  tmp_time_diff << endl;//"; now resp=" << resp << endl;
			found_time = true;
		}
		/*else if(resp.length() > 0)
		{
			stringstream tmp;
			tmp << resp;
			tmp >> tmp_time_diff; 
			DEBUG_STREAM << "Keithley2701::Scan() found last time, string= " << resp << " time number=" <<  tmp_time_diff << endl;
			//found_time = true;
		}*/
		
		pos_fin = resp.find(",");
		if(pos_fin != string::npos)
		{
			val = resp.substr(pos_ini, pos_fin-pos_ini);
			resp = resp.substr(pos_fin+1, resp.length()-pos_fin-1 );
			stringstream tmp;
			tmp << val;
			tmp >> num_ch; 
			DEBUG_STREAM << "Keithley2701::Scan() found ch string= " << val << " ch number=" <<  num_ch << endl;//"; now resp=" << resp << endl;
			found_ch = true;
		}
		else if(resp.length() > 0)
		{
			stringstream tmp;
			tmp << resp;
			tmp >> num_ch; 
			DEBUG_STREAM << "Keithley2701::Scan() found last ch, string= " << resp << " ch number=" <<  num_ch << endl;
			found_ch = true;
		}
		if(found_val && found_ch && found_time)
		{
			map<int,data_t>::iterator it = chan_map.find(num_ch);
			if(it != chan_map.end())
			{
				it->second.val = tmp_val;
				//transform global_read_time in s
				tmp_time = ((double)(global_read_time.tv_usec))/1000000;
				tmp_time += global_read_time.tv_sec;
				//DEBUG_STREAM << "Keithley2701::Scan() TIME: before tmp_time=" << tmp_time << endl;
				//printf("TIME: before tmp_time=%f\n",tmp_time);
				//sum read diff time in s
				tmp_time += (tmp_time_diff);
				//DEBUG_STREAM << "Keithley2701::Scan() TIME: after sum of " << tmp_time_diff << "s tmp_time=" << tmp_time << endl;
				//printf("TIME: after sum of %f tmp_time=%f\n",tmp_time_diff,tmp_time);
				//re-transform in struct timeval
				double fractpart, intpart;
				fractpart = modf (tmp_time , &intpart);
				//printf("fractpart=%f\n",fractpart);
				it->second.read_time.tv_sec = (__time_t)intpart;
				it->second.read_time.tv_usec = (__suseconds_t)(fractpart*1000000);
				//DEBUG_STREAM << "Keithley2701::Scan() TIME: before s= " << (long)global_read_time.tv_sec << " usec=" << (long)global_read_time.tv_usec << " after s= " << (long)it->second.read_time.tv_sec << " usec=" << (long)it->second.read_time.tv_usec << endl;
				//gettimeofday(&(it->second.read_time),NULL);
				if(!data_inited)
					data_inited = true;
			}
			else
				DEBUG_STREAM << "Keithley2701::Scan() ch=" << num_ch << " not found!" << endl;
		}
		i++;
	} while(pos_fin != string::npos);
	}
	//DEBUG_STREAM << "Keithley2701::Scan() chan_map.size=" << chan_map.size() << " chan_map=0x" << hex << (long)&(chan_map) << dec <<endl;
	return 0;
				
}

int Keithley2701::Configure()
{
	//DEBUG_STREAM << "Keithley2701::Configure(): entering... " << endl;
	vector<string>::iterator it;
	string resp,cmd;
	for(it=initString.begin(); it!=initString.end(); it++)
	{
		cmd = *it;
		DEBUG_STREAM << "Keithley2701::Configure() writing command: " << cmd << " read response=" << resp << endl;
		WriteRead(cmd,resp);
		abort_sleep((double)2.0*wr_sleep/1000);  //abort_sleep(sec)
		
	}
	return 0;
				
}


//+------------------------------------------------------------------
//
//	method:			Keithley2701::check_timeout
//
//	description:	method to execute "check_timeout"
//					Return -1 if timeout [msec.] exceeded
//
//-------------------------------------------------------------------
int Keithley2701::check_timeout(timeval time, double timeout)
{
	struct timeval timenow;
	gettimeofday(&timenow,NULL);
	
	if ((timenow.tv_sec -	time.tv_sec) > (timeout/1000)) {
		DEBUG_STREAM << "Keithley2701::check_timeout(): TIMEOUT! sec diff=" << (timenow.tv_sec - time.tv_sec) << " > timeout=" << (timeout/1000) << endl;
		return -1;
	}	
	if (((timenow.tv_sec -	time.tv_sec)*1000 - 
			(int)((double)(timenow.tv_usec -	time.tv_usec)/1000)) > timeout) {		
		return -1;
	}
	return 0;
	
}	/* end check_timeout() */	


//+------------------------------------------------------------------
//
//	method:			Keithley2701::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void Keithley2701::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.1); i++) {
		if (abortflag)
			break;
		omni_thread::sleep(0,100000000);
	}

}

#ifdef _OLD_ATTR
	/***************************************************************************
	
		ScalarAttributeDouble class
	  
	***************************************************************************/
	/*
	*   CTOR
	*/ 
	ScalarAttributeDouble::ScalarAttributeDouble(
		const std::string &attr_name,
		Tango::AttrWriteType rwtype,
		Tango::DeviceImpl *dev,
		int num_ch
		)
		
		:Attr(attr_name.c_str(),
		Tango::DEV_DOUBLE, rwtype
		)/*,
		
		LogAdapter(dev)*/
	{
		attr_read = new Tango::DevDouble();
		proxy_ = (Keithley2701 *)dev;
		cout << "Keithley2701::ScalarAttributeDouble contructor: proxy_=0x" <<hex<< long(proxy_) <<dec<< " num_ch=" << num_ch<<endl;
		num_ch_ = num_ch;
	}
	
	/*
	*   DTOR 
	*/  
	ScalarAttributeDouble::~ScalarAttributeDouble()
	{
		cout << "Keithley2701::ScalarAttributeDouble destructor: entering... num_ch=" << num_ch_ <<endl;
		if(attr_read!=0)
		{
			delete attr_read;
			attr_read = 0;
		}
		cout << "Keithley2701::ScalarAttributeDouble destructor: proxy_=0x" <<hex<< long(proxy_) <<dec<< " num_ch=" << num_ch_ <<endl;
		proxy_ = 0;
	}
	
	void ScalarAttributeDouble::init(
		const std::string &attr_name,
		Tango::AttrWriteType rwtype,
		Tango::DeviceImpl *dev,
		int num_ch
		)
	{
		proxy_ = (Keithley2701 *)dev;
		cout << "Keithley2701::ScalarAttributeDouble init: proxy_=0x" <<hex<< long(proxy_) <<dec<< " num_ch=" << num_ch<<endl;
		num_ch_ = num_ch;
	}
	
	/*
	*   read 
	*/  
	void ScalarAttributeDouble::read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{
		bool timeout = false;
		cout<<"Keithley2701::read: chan_map=0x" << hex << (long)&(proxy_->chan_map) << " proxy_=0x" << long(proxy_) << dec <<endl;
		if (proxy_ == 0)
		{
			//WARN_STREAM<<"Keithley2701::read: Proxy has not been created..."<<endl;
			cout << "Keithley2701::read: Proxy has not been created..."<<endl;
			att.set_quality(Tango::ATTR_INVALID);
		}
		else
		{
			map<int,data_t>::iterator it = proxy_->chan_map.find(num_ch_);
			if(it != proxy_->chan_map.end())
			{
				if (proxy_->check_timeout(it->second.read_time, proxy_->timeout) < 0) 
				{        
					//DEBUG_STREAM << "Keithley2701::read: "<< proxy_->attrName << " ch=" << num_ch_ << ": data is not refreshed" << endl;
					cout << "Keithley2701::read: "<< proxy_->attrName << " ch=" << num_ch_ << ": data is not refreshed" << endl;
					//attr.set_quality(Tango::ATTR_WARNING);
					timeout = true;					
				}
				*attr_read =  it->second.val;
				if(!timeout)
					//att.set_value(attr_read);
					att.set_value_date_quality(attr_read,it->second.read_time,Tango::ATTR_VALID, 1, 0, false);
				else
					att.set_value_date_quality(attr_read,it->second.read_time,Tango::ATTR_WARNING, 1, 0, false);
			}
			else
			{
				//INFO_STREAM<<"Keithley2701::read: Channel " << num_ch_ << " not found" <<endl;
				cout<<"Keithley2701::read: Channel " << num_ch_ << " not found, chan_map.size=" << proxy_->chan_map.size() << " chan_map=0x" << hex << (long)&(proxy_->chan_map) << dec <<endl;
				att.set_quality(Tango::ATTR_INVALID);
			}
		}
	}
#endif


}	//	namespace
