//=============================================================================
//
// file :        Keithley2701.h
//
// description : Include for the Keithley2701 class.
//
// project :	Keithley 2701
//
// $Author: claudio $
//
// $Revision: 1.7 $
//
// $Log: Keithley2701.h,v $
// Revision 1.7  2017-03-29 07:25:16  claudio
// ticket 1934: tango-9
//
// Revision 1.6  2013-08-23 10:44:58  graziano
// Device_4Impl
//
// Revision 1.5  2013-08-09 12:19:19  graziano
// fixed
//
// Revision 1.4  2010/12/15 08:57:48  graziano
// fixed socket timeout
//
// Revision 1.3  2009/11/13 07:56:42  graziano
// bug fixing
//
// Revision 1.2  2009/09/22 12:46:34  graziano
// bug fixing
//
// Revision 1.1  2009/09/22 12:10:18  graziano
// first commit
//
//
//
//		******************** README ******************** 
//		Example of configuration (Device Properties)
//
//		Channels	->		101,PreBuncher_1,C
//							102,PreBuncher_2,C
//							103,Buncher_1,C
//							104,Buncher_2,C
//							105,Sezione1_1,C
//							106,Sezione1_2,C
//							107,Sezione2_1,C
//							108,Sezione2_2,C
//							109,Volt_1,V
//							110,Volt_2,V
//
//		Conndevice	->		pertica/socket/g
//
//		InitString	->		*RST
//							TRAC:CLE
//							TRAC:CLE:AUTO ON
//							FORM:ELEM READ,TST,CHAN				(mandatory!)
//							CONF:TEMP (@101:108)				(temperature measurement)
//							TEMP:TRAN FRTD,(@101:108)			(PT100)
//							TEMP:FRTD:TYPE PT100,(@101:108)		(PT100)
//							CONF:DCV (@109:110)					(DCV measurement)
//							CALC:FORM MXB,(@110)				(math mX+b on ch 110)
//							CALC:KMATH:MMF 0.001,(@110)			(m param of mX+b)
//							CALC:KMATH:MBF 0.2,(@110)			(b param of mX+b)
//							CALC:STAT ON,(@110)					(enable math)
//							INIT:CONT OFF
//							TRIG:SOUR IMM
//							TRIG:COUN 1
//							SAMP:COUN 10
//							ROUT:SCAN (@101:110)
//							ROUT:SCAN:LSEL INT
//
//		ScanString	->		READ?
//
//		Th_sleep	->		10000
//
//		Wr_sleep	->		100
//
//		************************************************
//
// copyleft :    European Synchrotron Radiation Facility
//               BP 220, Grenoble 38043
//               FRANCE
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================
#ifndef _KEITHLEY2701_H
#define _KEITHLEY2701_H

#include <tango.h>
#include <sys/time.h>
//using namespace Tango;

/**
 * @author	$Author: claudio $
 * @version	$Revision: 1.7 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------
//units
#define	UNIT_DC_VOLTS		"VDC"
#define	UNIT_AC_VOLTS		"VAC"
#define	UNIT_DC_CURRENT		"ADC"
#define	UNIT_AC_CURRENT		"AAC"
#define UNIT_2WIRE_RES		"OHM"
#define UNIT_4WIRE_RES		"OHM4W"
#define UNIT_FREQ			"HZ"
#define UNIT_PERIOD			"SECS"
#define UNIT_TEMP_C			"C"
#define UNIT_TEMP_F			"F"
#define UNIT_TEMP_K			"K"

class readthread;

namespace Keithley2701_ns
{
	
typedef struct
{
	double val;
	string name;
	struct timeval read_time;
} data_t;
	
class ScalarAttributeDouble;
/**
 * Class Description:
 * Device server for Keithley 2701
 */

/*
 *	Device States Description:
*  Tango::RUNNING :  Temperature acquisition running
*  Tango::OFF :      Temperature acquisition stopped
*  Tango::FAULT :    Not communicating with device
 */


class Keithley2701: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------


	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
		Tango::DevDouble	*attr_ScalarAttribute_read;
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	Device proxy used to communicate with the instrument
 */
	string	conndevice;
/**
 *	Sleep time in ms between reads in thread
 */
	Tango::DevDouble	th_sleep;
/**
 *	Sleep time in ms between serial write of a command and serial read of the response
 */
	Tango::DevDouble	wr_sleep;
/**
 *	Commands to execute scan
 */
	vector<string>	scanString;
/**
 *	Array of channels: ch num, ch name
 */
	vector<string>	channels;
/**
 *	Basename of attributes
 */
	string	attrName;
/**
 *	Commands to init measurement
 */
	vector<string>	initString;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	Keithley2701(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	Keithley2701(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	Keithley2701(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~Keithley2701() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name Keithley2701 methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for ScalarAttribute acquisition result.
 */
	virtual void read_ScalarAttribute(Tango::Attribute &attr);
/**
 *	Read/Write allowed for ScalarAttribute attribute.
 */
	virtual bool is_ScalarAttribute_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for Start command.
 */
	virtual bool is_Start_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Stop command.
 */
	virtual bool is_Stop_allowed(const CORBA::Any &any);
/**
 * Start temperature acquisition
 *	@exception DevFailed
 */
	void	start();
/**
 * Sop temperature acquisition
 *	@exception DevFailed
 */
	void	stop();

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	



	Tango::DeviceProxy *devsocket;
	int abortflag;  // flag used to abort reading loop
	struct timeval global_read_time;
	int timeout;
	//double attr_val[64];	//max 64 channels
	bool scanning;
	int num_channels;
	map<int,data_t> chan_map;

	int WriteRead(string cmd, string &response);
	int Configure();
	int Scan();
	int check_timeout(timeval, double);	
	void abort_sleep(double time);
	omni_mutex *mutex;
	//int WriteRead(string cmd, string arg, data_t &rdata, bool response, bool parse=true);
	bool terminated;	//thread termonated

protected :	
	//	Add your own data members here
	//-----------------------------------------
	readthread *readloop;
	bool data_inited;
	long socket_timeout;
#ifdef _OLD_ATTR
	ScalarAttributeDouble* attr[64];	//max 64 channels
	vector<string> added_attr;
#endif
	int connection_status;
	Tango::DevDouble	*attr_val_read;
};

#ifdef _OLD_ATTR
////////////////////////////////////////////////////////////////////////////
//
//     class ScalarAttributeDouble
//
////////////////////////////////////////////////////////////////////////////
class ScalarAttributeDouble:public Tango::Attr/*, public Tango::LogAdapter*/
{	
public:

    //ctor
    ScalarAttributeDouble(
                        const std::string &attr_name,
						Tango::AttrWriteType rwtype,
                        Tango::DeviceImpl *dev,
                        int num_ch
                        );
  	
    //dtor
    ~ScalarAttributeDouble();
    
    void init(const std::string &attr_name,
    	Tango::AttrWriteType rwtype,
        Tango::DeviceImpl *dev,
        int num_ch
        );
	
    //read
    virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att);
	
protected:

    Tango::DevDouble *attr_read;
	Keithley2701 *proxy_;
	int num_ch_;
};
#endif
}	// namespace_ns

#endif	// _KEITHLEY2701_H
