//=============================================================================
//
// file :         Keithley2701Class.h
//
// description :  Include for the Keithley2701Class root class.
//                This class is represents the singleton class for
//                the Keithley2701 device class.
//                It contains all properties and methods which the 
//                Keithley2701 requires only once e.g. the commands.
//			
// project :      TANGO Device Server
//
// $Author: graziano $
//
// $Revision: 1.2 $
//
// $Log: Keithley2701Class.h,v $
// Revision 1.2  2009/09/24 07:43:25  graziano
// bug fixing
//
// Revision 1.1  2009/09/22 12:10:18  graziano
// first commit
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================

#ifndef _KEITHLEY2701CLASS_H
#define _KEITHLEY2701CLASS_H

#include <tango.h>
#include <Keithley2701.h>


namespace Keithley2701_ns
{//=====================================
//	Define classes for attributes
//=====================================
class ScalarAttributeAttrib: public Tango::Attr
{
public:
	ScalarAttributeAttrib():Attr("ScalarAttribute", Tango::DEV_DOUBLE, Tango::READ) {};
	ScalarAttributeAttrib(const char *name, long data_type, Tango::AttrWriteType w_type):Attr(name,data_type,w_type) {/*cout << "CTOR of " << name << endl;*/};
	~ScalarAttributeAttrib() {/*cout << "DTOR of " << get_name() << endl;*/};
	
	virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
	{(static_cast<Keithley2701 *>(dev))->read_ScalarAttribute(att);}
	virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
	{return (static_cast<Keithley2701 *>(dev))->is_ScalarAttribute_allowed(ty);}
};

//=========================================
//	Define classes for commands
//=========================================
class StopClass : public Tango::Command
{
public:
	StopClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	StopClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~StopClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<Keithley2701 *>(dev))->is_Stop_allowed(any);}
};



class StartClass : public Tango::Command
{
public:
	StartClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out,
				   const char        *in_desc,
				   const char        *out_desc,
				   Tango::DispLevel  level)
	:Command(name,in,out,in_desc,out_desc, level)	{};

	StartClass(const char   *name,
	               Tango::CmdArgType in,
				   Tango::CmdArgType out)
	:Command(name,in,out)	{};
	~StartClass() {};
	
	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
	{return (static_cast<Keithley2701 *>(dev))->is_Start_allowed(any);}
};



//
// The Keithley2701Class singleton definition
//

class
#ifdef WIN32
	__declspec(dllexport)
#endif
	Keithley2701Class : public Tango::DeviceClass
{
public:
//	properties member data

//	add your own data members here
//------------------------------------

public:
	Tango::DbData	cl_prop;
	Tango::DbData	cl_def_prop;
	Tango::DbData	dev_def_prop;

//	Method prototypes
	static Keithley2701Class *init(const char *);
	static Keithley2701Class *instance();
	~Keithley2701Class();
	Tango::DbDatum	get_class_property(string &);
	Tango::DbDatum	get_default_device_property(string &);
	Tango::DbDatum	get_default_class_property(string &);
	

	vector<string> default_Attribute_list;
	/// Function that diferentiates dynamic attributes from static ones
	bool isDynamicAttribute(string attrName)
	{
			bool isIn=false;
			for (int j=0; j<default_Attribute_list.size(); j++) {
				if (attrName==default_Attribute_list[j]) {
					isIn=true; j=default_Attribute_list.size();
					}
				}
			if(	(attrName == "SpectrumAttribute" ||
				attrName == "ScalarAttribute" ||
				attrName == "ImageAttribute" ||
				!isIn) &&
				(attrName!="State" &&
				attrName!="Status"))
				return true;
			else return false;
	}
	/// Function that removes dynamic attributes from the class attributes list
	void eraseDynamicAttributes(vector<Tango::Attr *> &att_list)
	{
		//cout << "eraseDynamicAttributes(vector<Tango::Attr *> &att_list) entering..."  << endl;
		/// Erasing undesired attributes
		vector<Tango::Attr *>::iterator a;
		for(a=att_list.begin(); a != att_list.end(); ++a)
		{
			if(isDynamicAttribute((*a)->get_name()))
			{
				cout << "Erasing attribute "<<(*a)->get_name()<<endl;
				delete (*a);
				att_list.erase(a);
				--a;
			} //else cout << "Standing attribute "<<(*i)->get_name()<<endl;
		}		
	}	
	
	/// Function that adds dynamic attributes to the devices attributes	 lists
	void addDynamicAttributes(const Tango::DevVarStringArray *devlist_ptr)
	{
		Tango::Util *tg = Tango::Util::instance();
		//cout << "DYNAMIC ATTRIBUTES GENERATION ... addDynamicAttributes(Tango::DevVarStringArray*)"  << endl;
		for (unsigned long i=0 ; i < devlist_ptr->length() ; i++)
		{
			int numOfAttr;
			Tango::DeviceImpl *mydev = tg->get_device_by_name(((string)(*devlist_ptr)[i]).c_str());
	
			Keithley2701 *myds = static_cast<Keithley2701 *> (mydev);
			numOfAttr = myds->channels.size();
	
			cout << "Device "<<((string)(*devlist_ptr)[i]).c_str()<<" will have " << numOfAttr << " Dynamic Attributes." << endl;
			//cout << "Names are "; for (int j = 0; j <numOfAttr ; j++) cout<<myds->channels[j]<<" "; cout<< endl;
	
			for (int j = 0; j <numOfAttr ; j++) {
					Tango::Attr *lval, *cspec;
					char attrname[256], strtmp[256];
					Tango::UserDefaultAttrProp attrprop;
					
					string num = "";
					int num_ch;
					string name = "";
					string unit = "";
					size_t pos = myds->channels[j].find(",");
					if(pos != string::npos)
					{
						num = myds->channels[j].substr(0, pos);
						name = myds->channels[j].substr(pos+1, myds->channels[j].length()-1 );
						pos = name.find(",");
						if(pos != string::npos)
						{
							unit = name.substr(pos+1, name.length()-1 );
							name = name.substr(0, pos);
						}
						stringstream tmp;
						num_ch;
						tmp << num;
						tmp >> num_ch;
					}
					//cout << "Parsed " << myds->channels[j] << " in num=" << num_ch << " name=" << name << endl;
	 				if (name=="") return;
					sprintf(attrname,"%s",name.c_str());
					cspec = new ScalarAttributeAttrib (attrname,Tango::DEV_DOUBLE,Tango::READ);
					sprintf(strtmp,"Dynamic Attribute %s reading channel %d ",attrname,num_ch);
					if(unit != "")
					{
						attrprop.set_unit(unit.c_str());
						attrprop.set_standard_unit("1.0");
						attrprop.set_display_unit(unit.c_str());
					}
					attrprop.set_format("%.9g");
					attrprop.set_description(strtmp);
					cspec->set_default_properties(attrprop);
					myds->add_attribute(cspec);
					}
		}
	}

	
protected:
	Keithley2701Class(string &);
	static Keithley2701Class *_instance;
	void command_factory();
	void get_class_property();
	void attribute_factory(vector<Tango::Attr *> &);
	void write_class_property();
	void set_default_property();

private:
	void device_factory(const Tango::DevVarStringArray *);
};


}	//	namespace Keithley2701_ns

#endif // _KEITHLEY2701CLASS_H
